<?php

namespace Drupal\janrain_connect_ui\Service;

use Drupal\Core\Config\ConfigFactory;
use GuzzleHttp\Exception\ClientException;
use Symfony\Component\Yaml\Yaml;
use GuzzleHttp\ClientInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Janrain Connect Flow Class.
 */
class JanrainConnectUiFlow {

  use StringTranslationTrait;

  /**
   * The HTTP client used to fetch remote definitions.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public function __construct(ClientInterface $http_client, ConfigFactory $config_factory) {
    $this->config = $config_factory->get('janrain_connect.settings');
    $this->httpClient = $http_client;
  }

  /**
   * Function to get Url Flow.
   */
  public function getUrlFlow($flow_data) {

    $url_flow = FALSE;

    // Check all properties.
    if (empty($flow_data['flow_url']) ||
        empty($flow_data['application_id']) ||
        empty($flow_data['lang_code']) ||
        empty($flow_data['flow_version']) ||
        empty($flow_data['flow_name'])) {

      return FALSE;
    }

    // Trim for remove extras spaces.
    foreach ($flow_data as $key => $flow) {
      $flow_data[$key] = trim($flow);
    }

    $url_flow = implode(':', $flow_data);

    return $url_flow;
  }

  /**
   * Function to get Flow Response.
   */
  public function getFlowResponse($flow) {

    $flow_timeout = $this->config->get('flow_timeout');

    // Make an HTTP Request with Guzzle.
    $client = $this->httpClient;

    try {
      $request = $client->get($flow, [
        'timeout' => $flow_timeout,
      ]);

      $flow = (string) $request->getBody();

      // Check result by flow.
      if (empty($flow)) {

        drupal_set_message($this->t('Flow returned an invalid response'), 'error');

        drupal_set_message($this->t('Flow URL: @flow_url', [
          '@flow_url' => $flow,
        ]), 'error');

        return FALSE;
      }
    }
    catch (ClientException $e) {

      drupal_set_message($this->t('Client Error: @get_status_code', [
        '@get_status_code' => $e->getResponse()->getStatusCode(),
      ]), 'error');

      drupal_set_message($this->t('Flow URL: @flow_url', [
        '@flow_url' => $flow,
      ]), 'error');

      return FALSE;
    }

    // Get flow status.
    $flow_status = strstr($flow, 'function () { janrain.capture.ui.render(', TRUE);
    $flow_status = str_replace('janrain.capture.ui.handleCaptureResponse(', '', $flow_status);
    $flow_status = substr($flow_status, 0, -1);

    $flow_status_array = Yaml::parse($flow_status);
    $flow_status_yaml = Yaml::dump($flow_status_array, 2, 2);

    // Get flow data.
    $flow_data = strstr($flow, 'function () { janrain.capture.ui.render(');
    $flow_data = str_replace('function () { janrain.capture.ui.render(', '', $flow_data);
    $flow_data = trim($flow_data);
    $flow_data = substr($flow_data, 0, -6);

    $flow_data_array = Yaml::parse($flow_data);
    $flow_data_yaml = Yaml::dump($flow_data_array, 10, 2);

    // Flow yaml.
    $flow_yaml = $flow_status_yaml . $flow_data_yaml;

    return $flow_yaml;
  }

}
